// ============================================================
// Project            :  Mi Pueblito
// File               :  .\game\gameScripts\highScores.cs
// Copyright          :  
// Author             :  Pier
// Created on         :  Wednesday, September 05, 2007 6:46 PM
//
// Editor             :  Codeweaver v. 1.2.2594.2497
//
// Description        :  Funciones para manejar una base de 
//                    : datos con los scores mas altos. 
//                    :  
// ============================================================


// Creamos una nueva table de scores, con un maximo de jugadores de 5
function NewHighScores() {
   %obj = new ScriptObject() {
      class = "HighScores";
      MaxCount = 5;
   };
   %obj.Scores = new SimSet();
   return(%obj);
}


// Llenamos la base de datos con datos default
function HighScores::createDefaultTable(%this) {
   for (%i= 0; %i < %this.MaxCount; %i++) {
      %newScore = new SimObject() {
         playerName= "Guillen";
         playerScore= 60*(5 -%i);
      };
      %this.Scores.add(%newScore);
   }
}


// Agregamos al juagador a la tabla de posiciones, en caso de que califique
function HighScores::add(%this, %player) {
  
   // Revisamos que los parametros sean validos
   if ((!isObject(%player)) || (%player.PlayerName $= "") || (%player.PlayerScore $= "")) {
      echo("Wrong parameters when adding player to highscore");
      echo(%this SPC %player);
      return (false);
   }

   // Buscamos en que posicion quedo
   %position= %this.MaxCount;
   for (%i = %this.MaxCount -1; %i>= 0; %i--) {
      if (%this.Scores.getObject(%i).PlayerScore < %player.PlayerScore) {
         %position= %i;
      } else {
         break;
      }
   }

   // Recorremos a los demas
   for (%i = %this.MaxCount -1; %i> %position; %i--) {
      %this.Scores.getObject(%i).PlayerName  = %this.Scores.getObject(%i -1).PlayerName;
      %this.Scores.getObject(%i).PlayerScore = %this.Scores.getObject(%i -1).PlayerScore;
   }

   // Y lo agregamos en esa posicion
   if (%position!= %this.MaxCount) {
      %this.Scores.getObject(%position).PlayerName  = %player.PlayerName;
      %this.Scores.getObject(%position).PlayerScore = %player.PlayerScore;
   }
   return (true);
}


// Regrea el score de la posicion que le mandamos
function HighScores::getScore(%this, %rank) {
   if (%this.Scores.getCount() < %rank - 1) {
      return -1;
   }
   return (%this.Scores.getObject(%rank).PlayerScore);
}


// Regresa el nombre del jugador de la posicion que le mandamos
function HighScores::getPlayer(%this, %rank) {
   if (%this.Scores.getCount() < %rank - 1) {
      return -1;
   }
   return (%this.Scores.getObject(%rank).PlayerName);
}


// Regresa los datos del jugador de la posicion que le mandamos
function HighScores::getPlayerScore(%this, %rank) {
   if (%this.Scores.getCount() < %rank - 1) {
      return -1;
   }
   return %this.Scores.getObject(%rank);
}


// Regresa en que posicion se encuentra el jugador
function HighScores::getRank(%this, %player) {
   for(%i = 0; %i < %this.Scores.getCount(); %i++) {
      if (%this.Scores.getObject(%i).PlayerName $= %player.PlayerName) {
         return %i;
      }
   }
}


// Mandamos guardar la base de datos en %path
function HighScores::save(%this, %path) {

echo ("S>" SPC getMyGamesModPaths() @ %path);
   %file = new FileObject();
   if (%file.openForWrite( getMyGamesModPaths() @ %path)) {
      for (%i = 0; %i < %this.MaxCount; %i++) {
         %file.writeLine(%this.Scores.getObject(%i).PlayerName SPC %this.Scores.getObject(%i).PlayerScore);
      }
      echo("File Written");
   } else {
      error("File is not open for writing");
   }
   %file.close();
   %file.delete();
}


// Carga una base de datos desde %path
function HighScores::load(%this, %path) {

echo ("L>" SPC getMyGamesModPaths() @ %path);
   %file = new FileObject();
   if (%file.openForRead( getMyGamesModPaths() @ %path)) {
      for (%i = 0; %i < %this.MaxCount; %i++) {
         %line = %file.readLine();
echo(%line);         
         %newScore = new SimObject() {
            %getWordCount(text);
            playerName = getWord(%line, 0);
            playerScore= getWord(%line, 1);
         };
         %this.Scores.add(%newScore);
echo(%newScore.playerName SPC %newScore.playerScore);         
         //echo("line" @ %i @ " = " @ %line);
      }
      /*
      %i= 1;
      while (!%file.isEof()) {
         %line = %file.readLine();
         echo("line" @ %i @ " = " @ %line);
         %i++;
      }
      */
   } else {
      error("Can't Open Highscore Table. Creating a new one.");
      %this.createDefaultTable();
   }
   %file.close();
   %file.delete();
}


// Imprime los datos de la tabla en consola
function HighScores::debug(%this) {
   if(!isObject(%this.Scores)) {
      echo("No Scores in Stack");
      return;
   }

   for(%i=0; %i< %this.Scores.getCount(); %i++) {
      echo("  Player: " @ %this.Scores.getObject(%i).PlayerName);
      echo("  Scores: " @ %this.Scores.getObject(%i).PlayerScore);
      echo(" ");
   }
}